@@ -1,9 +1,11 @@
 import abc
+import ipaddress
 import json
 import queue
 import re
 import threading
 import time
+import urllib.parse
 from collections import defaultdict
 from contextlib import contextmanager
 from contextlib import suppress
@@ -21,9 +23,9 @@ from typing import Pattern
 from typing import Tuple
 from typing import Union
 
-import werkzeug.urls
+import werkzeug.http
+from werkzeug.datastructures import Authorization
 from werkzeug.datastructures import MultiDict
-from werkzeug.http import parse_authorization_header
 from werkzeug.serving import make_server
 from werkzeug.wrappers import Request
 from werkzeug.wrappers import Response
@@ -32,10 +34,12 @@ URI_DEFAULT = ""
 METHOD_ALL = "__ALL"
 
 HEADERS_T = Union[
-    Mapping[str, Union[str, int, Iterable[Union[str, int]]]],
-    Iterable[Tuple[str, Union[str, int]]],
+    Mapping[str, Union[str, Iterable[str]]],
+    Iterable[Tuple[str, str]],
 ]
 
+HVMATCHER_T = Callable[[str, Optional[str], str], bool]
+
 
 class Undefined:
     def __repr__(self):
@@ -130,7 +134,10 @@ class HeaderValueMatcher:
 
     @staticmethod
     def authorization_header_value_matcher(actual: Optional[str], expected: str) -> bool:
-        return parse_authorization_header(actual) == parse_authorization_header(expected)
+        callable = getattr(Authorization, "from_header", None)
+        if callable is None:  # Werkzeug < 2.3.0
+            callable = werkzeug.http.parse_authorization_header
+        return callable(actual) == callable(expected)
 
     @staticmethod
     def default_header_value_matcher(actual: Optional[str], expected: str) -> bool:
@@ -213,7 +220,7 @@ class MappingQueryMatcher(QueryMatcher):
         self.query_dict = query_dict
 
     def get_comparing_values(self, request_query_string: bytes) -> tuple:
-        query = werkzeug.urls.url_decode(request_query_string)
+        query = MultiDict(urllib.parse.parse_qsl(request_query_string.decode("utf-8")))
         if isinstance(self.query_dict, MultiDict):
             return (query, self.query_dict)
         else:
@@ -288,6 +295,13 @@ class RequestMatcher:
         specified in the request. If multiple values specified for a given key, the first
         value will be used. If multiple values needed to be handled, use ``MultiDict``
         object from werkzeug.
+    :param header_value_matcher: :py:class:`HeaderValueMatcher` that matches
+        values of headers, or a ``Callable[[str, Optional[str], str], bool]``
+        receiving the header key (from `headers`), header value (or `None`) and the expected
+        value (from `headers`) and should return ``True`` if the header matches, ``False`` otherwise.
+    :param json: a python object (eg. a dict) whose value will be compared to the request body after it
+        is loaded as json. If load fails, this matcher will be failed also. *Content-Type* is not checked.
+        If that's desired, add it to the headers parameter.
     """
 
     def __init__(
@@ -298,10 +312,9 @@ class RequestMatcher:
         data_encoding: str = "utf-8",
         headers: Optional[Mapping[str, str]] = None,
         query_string: Union[None, QueryMatcher, str, bytes, Mapping] = None,
-        header_value_matcher: Optional[HeaderValueMatcher] = None,
+        header_value_matcher: Optional[HVMATCHER_T] = None,
         json: Any = UNDEFINED,
     ):
-
         if json is not UNDEFINED and data is not None:
             raise ValueError("data and json parameters are mutually exclusive")
 
@@ -321,7 +334,10 @@ class RequestMatcher:
         self.data = data
         self.data_encoding = data_encoding
 
-        self.header_value_matcher = HeaderValueMatcher() if header_value_matcher is None else header_value_matcher
+        self.header_value_matcher: HVMATCHER_T = HeaderValueMatcher()
+
+        if header_value_matcher is not None:
+            self.header_value_matcher = header_value_matcher
 
     def __repr__(self):
         """
@@ -610,6 +626,9 @@ class HTTPServerBase(abc.ABC):  # pylint: disable=too-many-instance-attributes
         self.ssl_context = ssl_context
         self.no_handler_status_code = 500
 
+    def __repr__(self):
+        return f"<{self.__class__.__name__} host={self.host} port={self.port}>"
+
     def clear(self):
         """
         Clears and resets the state attributes of the object.
@@ -650,6 +669,11 @@ class HTTPServerBase(abc.ABC):  # pylint: disable=too-many-instance-attributes
         This basically means that it prepends the string ``http://$HOST:$PORT/`` to the `suffix` parameter
         (where $HOST and $PORT are the parameters given to the constructor).
 
+        When host is an IPv6 address, the required square brackets will be added
+        to it, forming a valid URL.
+
+        When SSL or TLS is in use, the protocol of the returned URL will be ``https``.
+
         :param suffix: the suffix which will be added to the base url. It can start with ``/`` (slash) or
             not, the url will be the same.
         :return: the full url which refers to the server
@@ -663,7 +687,9 @@ class HTTPServerBase(abc.ABC):  # pylint: disable=too-many-instance-attributes
         else:
             protocol = "https"
 
-        return "{}://{}:{}{}".format(protocol, self.host, self.port, suffix)
+        host = self.format_host(self.host)
+
+        return "{}://{}:{}{}".format(protocol, host, self.port, suffix)
 
     def create_matcher(self, *args, **kwargs) -> RequestMatcher:
         """
@@ -837,6 +863,24 @@ class HTTPServerBase(abc.ABC):  # pylint: disable=too-many-instance-attributes
         if self.is_running():
             self.stop()
 
+    @staticmethod
+    def format_host(host):
+        """
+        Formats a hostname so it can be used in a URL.
+        Notably, this adds brackets around IPV6 addresses when
+        they are missing.
+        """
+        try:
+            ipaddress.IPv6Address(host)
+            is_ipv6 = True
+        except ValueError:
+            is_ipv6 = False
+
+        if is_ipv6 and not host.startswith("[") and not host.endswith("]"):
+            return f"[{host}]"
+
+        return host
+
 
 class HTTPServer(HTTPServerBase):  # pylint: disable=too-many-instance-attributes
     """
@@ -910,7 +954,7 @@ class HTTPServer(HTTPServerBase):  # pylint: disable=too-many-instance-attribute
         data_encoding: str = "utf-8",
         headers: Optional[Mapping[str, str]] = None,
         query_string: Union[None, QueryMatcher, str, bytes, Mapping] = None,
-        header_value_matcher: Optional[HeaderValueMatcher] = None,
+        header_value_matcher: Optional[HVMATCHER_T] = None,
         handler_type: HandlerType = HandlerType.PERMANENT,
         json: Any = UNDEFINED,
     ) -> RequestHandler:
@@ -944,7 +988,10 @@ class HTTPServer(HTTPServerBase):  # pylint: disable=too-many-instance-attribute
             specified in the request. If multiple values specified for a given key, the first
             value will be used. If multiple values needed to be handled, use ``MultiDict``
             object from werkzeug.
-        :param header_value_matcher: :py:class:`HeaderValueMatcher` that matches values of headers.
+        :param header_value_matcher: :py:class:`HeaderValueMatcher` that matches
+            values of headers, or a ``Callable[[str, Optional[str], str], bool]``
+            receiving the header key (from `headers`), header value (or `None`) and the expected
+            value (from `headers`) and should return ``True`` if the header matches, ``False`` otherwise.
         :param handler_type: type of handler
         :param json: a python object (eg. a dict) whose value will be compared to the request body after it
             is loaded as json. If load fails, this matcher will be failed also. *Content-Type* is not checked.
@@ -982,7 +1029,7 @@ class HTTPServer(HTTPServerBase):  # pylint: disable=too-many-instance-attribute
         data_encoding: str = "utf-8",
         headers: Optional[Mapping[str, str]] = None,
         query_string: Union[None, QueryMatcher, str, bytes, Mapping] = None,
-        header_value_matcher: Optional[HeaderValueMatcher] = None,
+        header_value_matcher: Optional[HVMATCHER_T] = None,
         json: Any = UNDEFINED,
     ) -> RequestHandler:
         """
@@ -1004,7 +1051,10 @@ class HTTPServer(HTTPServerBase):  # pylint: disable=too-many-instance-attribute
             specified in the request. If multiple values specified for a given key, the first
             value will be used. If multiple values needed to be handled, use ``MultiDict``
             object from werkzeug.
-        :param header_value_matcher: :py:class:`HeaderValueMatcher` that matches values of headers.
+        :param header_value_matcher: :py:class:`HeaderValueMatcher` that matches
+            values of headers, or a ``Callable[[str, Optional[str], str], bool]``
+            receiving the header key (from `headers`), header value (or `None`) and the expected
+            value (from `headers`) and should return ``True`` if the header matches, ``False`` otherwise.
         :param json: a python object (eg. a dict) whose value will be compared to the request body after it
             is loaded as json. If load fails, this matcher will be failed also. *Content-Type* is not checked.
             If that's desired, add it to the headers parameter.
@@ -1034,7 +1084,7 @@ class HTTPServer(HTTPServerBase):  # pylint: disable=too-many-instance-attribute
         data_encoding: str = "utf-8",
         headers: Optional[Mapping[str, str]] = None,
         query_string: Union[None, QueryMatcher, str, bytes, Mapping] = None,
-        header_value_matcher: Optional[HeaderValueMatcher] = None,
+        header_value_matcher: Optional[HVMATCHER_T] = None,
         json: Any = UNDEFINED,
     ) -> RequestHandler:
         """
@@ -1056,7 +1106,10 @@ class HTTPServer(HTTPServerBase):  # pylint: disable=too-many-instance-attribute
             specified in the request. If multiple values specified for a given key, the first
             value will be used. If multiple values needed to be handled, use ``MultiDict``
             object from werkzeug.
-        :param header_value_matcher: :py:class:`HeaderValueMatcher` that matches values of headers.
+        :param header_value_matcher: :py:class:`HeaderValueMatcher` that matches
+            values of headers, or a ``Callable[[str, Optional[str], str], bool]``
+            receiving the header key (from `headers`), header value (or `None`) and the expected
+            value (from `headers`) and should return ``True`` if the header matches, ``False`` otherwise.
         :param json: a python object (eg. a dict) whose value will be compared to the request body after it
             is loaded as json. If load fails, this matcher will be failed also. *Content-Type* is not checked.
             If that's desired, add it to the headers parameter.
